using System;
using System.Collections.Generic;
using System.Text;
using System.Net.Sockets;
using System.IO;
using System.Net.Security;
using System.Security.Cryptography.X509Certificates;
using System.Security.Authentication;

namespace LeastPrivilege
{
    class Client
    {
        static void Main(string[] args)
        {
            string machineName = "localhost";
            
            // erwartete subject name des server zertifikats
            string serverSN = "Dominick Baier";

            // client zertifikat
            X509Certificate2Collection clientCerts = new X509Certificate2Collection(getClientCert());

            TcpClient client = new TcpClient();
            client.Connect(machineName, 4242);

            // wrappen des streams - spezifizieren der zertifikats-prfroutine
            SslStream ssl = new SslStream(
                client.GetStream(),
                false,
                CertificateValidationCallback,
                CertificateSelectionCallback);

            // SSL handshake starten 
            ssl.AuthenticateAsClient(
                serverSN,                   // server subject name
                clientCerts,                // client zertifikat
                SslProtocols.Tls,           // SSL version
                true);                      // server zertifikate gegen rckruf-liste prfen

            showSslInfo(machineName, ssl, true);

            TextWriter writer = new StreamWriter(ssl);

            writer.WriteLine("Hallo SSL-Server");
            writer.Flush();

            client.Close();

            Console.ReadKey();
        }

        // remote Zertifikat prfen
        static bool CertificateValidationCallback(object sender, X509Certificate certificate, X509Chain chain, SslPolicyErrors sslPolicyErrors)
        {
            if (sslPolicyErrors != SslPolicyErrors.None)
            {
                Console.WriteLine("SSL Certificate Validation Error!");
                Console.Write(certificate.Subject);
                Console.WriteLine(sslPolicyErrors.ToString());
                return false;
            }
            else
                return true;
        }

        // callback um das client zertifikate auszuwhlen (kann im ctor von SslStream angegeben werden)
        // diese methode erlaubt mehr flexibilitt als das zertifikat vor der authentifizierung festzulegen
        static X509Certificate CertificateSelectionCallback(object sender, string targetHost, X509CertificateCollection localCertificates, X509Certificate remoteCertificate, string[] acceptableIssuers)
        {
            Console.WriteLine("selecting cert for: {0}", targetHost);
            Console.WriteLine("acceptable issuers: \n{0}", string.Join(", \n", acceptableIssuers));
            return getClientCert();
        }


        private static void showSslInfo(string serverName, SslStream sslStream, bool verbose)
        {
            Console.WriteLine("\n\nSSL Connect Report for : {0}\n", serverName);
            Console.WriteLine("Is Authenticated:            {0}", sslStream.IsAuthenticated);
            Console.WriteLine("Is Encrypted:                {0}", sslStream.IsEncrypted);
            Console.WriteLine("Is Signed:                   {0}", sslStream.IsSigned);
            Console.WriteLine("Is Mutually Authenticated:   {0}\n", sslStream.IsMutuallyAuthenticated);

            Console.WriteLine("Hash Algorithm:              {0}", sslStream.HashAlgorithm);
            Console.WriteLine("Hash Strength:               {0}", sslStream.HashStrength);
            Console.WriteLine("Cipher Algorithm:            {0}", sslStream.CipherAlgorithm);
            Console.WriteLine("Cipher Strength:             {0}\n", sslStream.CipherStrength);

            Console.WriteLine("Key Exchange Algorithm:      {0}", sslStream.KeyExchangeAlgorithm);
            Console.WriteLine("Key Exchange Strength:       {0}\n", sslStream.KeyExchangeStrength);
            Console.WriteLine("SSL Protocol:                {0}", sslStream.SslProtocol);

            showCertificateInfo(sslStream.RemoteCertificate, verbose);
        }

        private static void showCertificateInfo(X509Certificate remoteCertificate, bool verbose)
        {
            Console.WriteLine("Certficate Information for:\n{0}\n", remoteCertificate.Subject);
            Console.WriteLine("Valid From:      \n{0}", remoteCertificate.GetEffectiveDateString());
            Console.WriteLine("Valid To:        \n{0}", remoteCertificate.GetExpirationDateString());
            Console.WriteLine("Certificate Format:     \n{0}\n", remoteCertificate.GetFormat());

            Console.WriteLine("Issuer Name:     \n{0}", remoteCertificate.Issuer);

            if (verbose)
            {
                Console.WriteLine("Serial Number:   \n{0}", remoteCertificate.GetSerialNumberString());
                Console.WriteLine("Hash:            \n{0}", remoteCertificate.GetCertHashString());
                Console.WriteLine("Key Algorithm:   \n{0}", remoteCertificate.GetKeyAlgorithm());
                Console.WriteLine("Key Algorithm Parameters:     \n{0}", remoteCertificate.GetKeyAlgorithmParametersString());
                Console.WriteLine("Public Key:     \n{0}", remoteCertificate.GetPublicKeyString());
            }
        }

        // zertifikat aus speicher lesen
        private static X509Certificate2 getClientCert()
        {
            X509Store store = null;

            try
            {
                store = new X509Store(StoreName.My, StoreLocation.CurrentUser);
                store.Open(OpenFlags.ReadOnly);
                X509Certificate2Collection cert = store.Certificates.Find(
                    X509FindType.FindBySubjectKeyIdentifier,
                    "21f2bf447298e83056a69eb02ebe9085ed97f10a",
                    true);

                return cert[0];
            }
            finally
            {
                store.Close();
            }
        }
    }
}
