using System.Security.Cryptography.X509Certificates;
using System.Net.Sockets;
using System.Net;
using System.Net.Security;
using System.Security.Authentication;
using System.IO;
using System;

namespace LeastPrivilege
{
    class Server
    {
        static void Main(string[] args)
        {
            X509Certificate2 serverCert = getServerCert();

            TcpListener listener = new TcpListener(IPAddress.Any, 4242);
            listener.Start();

            TcpClient client = listener.AcceptTcpClient();

            // network stream wrappen
            SslStream ssl = new SslStream(client.GetStream(),true);

            // SSL handshake starten
            ssl.AuthenticateAsServer(
                serverCert,             // server zertifikat
                true,                   // client authentifizierung erzwingen
                SslProtocols.Tls,       // gewnschte SSL version
                true);                  // client zertifikat gegen rckruf-liste prfen

            showSslInfo(ssl, true);

            TextReader reader = new StreamReader(ssl);
            string message = reader.ReadLine();

            Console.WriteLine(message);
        }

        // zertifikat aus speicher lesen
        private static X509Certificate2 getServerCert()
        {
            X509Store store = null;

            try
            {
                store = new X509Store(StoreName.My, StoreLocation.CurrentUser);
                store.Open(OpenFlags.ReadOnly);
                X509Certificate2Collection cert = store.Certificates.Find(
                    X509FindType.FindBySubjectKeyIdentifier,
                    "7bf3fe81e7185387b8ae687d36754a96ce790fe6",
                    true);

                return cert[0];
            }
            finally
            {
                store.Close();
            }
        }

        private static void showSslInfo(SslStream sslStream, bool verbose)
        {
            Console.WriteLine("Is Authenticated:            {0}", sslStream.IsAuthenticated);
            Console.WriteLine("Is Encrypted:                {0}", sslStream.IsEncrypted);
            Console.WriteLine("Is Signed:                   {0}", sslStream.IsSigned);
            Console.WriteLine("Is Mutually Authenticated:   {0}\n", sslStream.IsMutuallyAuthenticated);

            Console.WriteLine("Hash Algorithm:              {0}", sslStream.HashAlgorithm);
            Console.WriteLine("Hash Strength:               {0}", sslStream.HashStrength);
            Console.WriteLine("Cipher Algorithm:            {0}", sslStream.CipherAlgorithm);
            Console.WriteLine("Cipher Strength:             {0}\n", sslStream.CipherStrength);

            Console.WriteLine("Key Exchange Algorithm:      {0}", sslStream.KeyExchangeAlgorithm);
            Console.WriteLine("Key Exchange Strength:       {0}\n", sslStream.KeyExchangeStrength);
            Console.WriteLine("SSL Protocol:                {0}", sslStream.SslProtocol);

            showCertificateInfo(sslStream.RemoteCertificate, verbose);
        }

        private static void showCertificateInfo(X509Certificate remoteCertificate, bool verbose)
        {
            Console.WriteLine("Certficate Information for:\n{0}\n", remoteCertificate.Subject);
            Console.WriteLine("Valid From:      \n{0}", remoteCertificate.GetEffectiveDateString());
            Console.WriteLine("Valid To:        \n{0}", remoteCertificate.GetExpirationDateString());
            Console.WriteLine("Certificate Format:     \n{0}\n", remoteCertificate.GetFormat());

            Console.WriteLine("Issuer Name:     \n{0}", remoteCertificate.Issuer);

            if (verbose)
            {
                Console.WriteLine("Serial Number:   \n{0}", remoteCertificate.GetSerialNumberString());
                Console.WriteLine("Hash:            \n{0}", remoteCertificate.GetCertHashString());
                Console.WriteLine("Key Algorithm:   \n{0}", remoteCertificate.GetKeyAlgorithm());
                Console.WriteLine("Key Algorithm Parameters:     \n{0}", remoteCertificate.GetKeyAlgorithmParametersString());
                Console.WriteLine("Public Key:     \n{0}", remoteCertificate.GetPublicKeyString());
            }
        }
    }
}
